(***************************************************************************)
(*                              MPC 2012                                   *)
(*                                                                         *)
(*                                                                         *)
(* Authors: Louis Mandel (louis.mandel@lri.fr)                             *)
(*          Florence Plateau (florence.plateau@lri.fr)                     *)
(*                                                                         *)
(* Creation date: April 2012                                               *)
(*                                                                         *)
(***************************************************************************)

(** Definition of clock types and subtyping constraints.

    @author Louis Mandel
    @author Florence Plateau
*)

open Misc
open Pbword

(** {2 Data structures} *)

(** Clock type varaible: ['a_n]. *)
type clock_type_variable = Alpha of int

(** Clock type: [ct ::= 'a | ct on p]. *)
type clock_type =
  | CT_var of clock_type_variable
  | CT_on of clock_type * periodic_binary_word

(** Subtyping constraint: ['a_x on p_1 on ... <: 'a_y on p_2 on ...]. *)
type subtype_cstr = Subtype of clock_type * clock_type


(** {2 Printing functions} *)

let fprint_clock_type_var ff (Alpha n) =
  Format.fprintf ff "'a_%d" n

let rec fprint_clock_type ff ct =
  match ct with
  | CT_var alpha -> fprint_clock_type_var ff alpha
  | CT_on (ct', p) ->
      Format.fprintf ff "%a on %a"
        fprint_clock_type ct'
        fprint_pbw p

let fprint_subtyping_cstr ff (Subtype (ct_x, ct_y)) =
  Format.fprintf ff "@[%a@] <: @[%a@]"
    fprint_clock_type ct_x
    fprint_clock_type ct_y

let fprint_subtyping_cstrs ff cstrs =
  Format.fprintf ff "{@[<hov 1> %a@] }"
    (fprint_list ";@\n" fprint_subtyping_cstr) cstrs

